//--------------------------------------------------
// NETVSL_ArgHandler.cs
// Part of Diagramantics NETVSL .NET-to-VSL interface for Visio
// Provided for use with the book:
// Microsoft Visio 2003 Developer's Survival Pack
// Copyright 2003
// Graham Wideman
// Permission granted to owners of the book to incorporate into your own 
// code, and to deploy that code freely.
// 
// -----------------------------
// Purpose:
// Implements a TArgHandler class to parse the arguments that
// Visio passes in the RUN method.
// Very pedestrian implementation, no doubt you could do better :-)
// -----------------------------
//
// ----------------------------------------------
//     I M P O R T A N T
// ----------------------------------------------
// -- Probably no need to modify this file.
//
// -- Simply include this file it in your C# project, by adding a 
//    *Link* to this file in your project in the Solution Explorer. 
//    Ie: Use Add > Add Existing Item and in the open-file dialog
//    use the Open button's drop down to select Link. It's fiddly,
//    you have to select but not double click the filename first.
//
// -- See samples to see how to use
//
// -----------------------------------------------
//
// Revision History:
// 
// 2004-02-14 GW First ,NET release version
// 2001-xx-xx GW VB6 version
//
//------------------------------------------------

using System;
using System.Collections;

namespace NETVSL_Common
{
	/// <summary>
	/// Summary description for NETVSL_ArgHandler.
	/// </summary>
	internal class TArgHandler
	{
    private enum TParseStep {
      ps_None     = 1,
      ps_InKey       ,
      ps_AfterKey    ,  
      ps_InVal
    }

    private class TParseState {
      public TParseStep   ParseStep;
      public String       Key;
      public String       Item;
      public Boolean      Quoted;
    }

    private class TListItem {
      public String   Key;
      public String   Value;
    }

    private ArrayList AList;

		public TArgHandler() // ctor
		{
      AList = new ArrayList();
		}

    //------------------------------------
    private void ParseState_Reset(ref TParseState ParseState) { 
    //------------------------------------
      ParseState.Key         = "";
      ParseState.Item        = "";
      ParseState.ParseStep   = TParseStep.ps_None;
      ParseState.Quoted      = false;
    }

    //------------------------------------
    private void ParseState_Enlist(ref TParseState ParseState) { 
    //------------------------------------
      TListItem AListItem = new TListItem();
      AListItem.Key        = ParseState.Key;
      AListItem.Value      = ParseState.Item;
      AList.Add(AListItem);
      ParseState_Reset(ref ParseState);
    }

    //------------------------------------
    public void ParseArgs(String ArgStr) { 
    //------------------------------------
      TParseState  ParseState = new TParseState();
      char C, NextC;     
      
      ParseState_Reset(ref ParseState);
      AList.Clear();
      
      String MyArgStr = ArgStr;
      if (MyArgStr == null) MyArgStr = "";
  
      int L = MyArgStr.Length;  // L = Length
      int P = -1;             // P = Position, 0 based in C#
      C = ' ';
  
      while (P < L-1) {
        P++;
        C = MyArgStr[P];

        if (P < L-1) {
          NextC = MyArgStr[P + 1];
        } else {
          NextC = ' ';
        }

        switch (ParseState.ParseStep) 
        {
          //---------------
          case TParseStep.ps_None:    
            switch (C) 
            {
              case ' ': 
              case '/': 
                break ; // do nothing, ignore flag indicator
              default: 
                ParseState.Key        = "" + Char.ToUpper(C);
                ParseState.ParseStep  = TParseStep.ps_InKey;
                break;
            }
            break;
          //---------------
          case TParseStep.ps_InKey:
            switch (C) 
            {
              case ' ':        // space after key means no value follows
                ParseState_Enlist(ref ParseState);
                break;
              case '=':        // value follows, continue
                ParseState.ParseStep  = TParseStep.ps_AfterKey;
                break;
              default:
                ParseState.Key = ParseState.Key + Char.ToUpper(C);
                break;
            }
            break;
          //---------------
          case TParseStep.ps_AfterKey:
            switch(C) 
            {
              case ' ': 
                break;    // do nothing
              case '"' :          // quoted item (a lone double-quote)
                ParseState.ParseStep = TParseStep.ps_InVal;
                ParseState.Quoted    = true;
                break;
              default:
                ParseState.Item      = ParseState.Item + C;
                ParseState.ParseStep = TParseStep.ps_InVal;
                ParseState.Quoted    = false;
                break;
            }
            break;
          //---------------
          case TParseStep.ps_InVal:
            switch (C) 
            {
              case ' ':
                if (ParseState.Quoted) {
                  ParseState.Item = ParseState.Item + C;
                } else {
                  ParseState_Enlist(ref ParseState);
                }
                break;
              case '"':
                if (ParseState.Quoted) {    
                  ParseState_Enlist(ref ParseState);
                } else {  
                  ParseState.Item = ParseState.Item + C;
                }  
                break;
              default:
                ParseState.Item = ParseState.Item + C;
                break;
            }
            break;
        } // outer switch   
      } // while
      
      if (ParseState.Key.Length > 0) {
        ParseState_Enlist(ref ParseState);
      }  
    }

    //------------------------------------
    public int ArgCount() { 
    //------------------------------------
      return AList.Count;
    }
    
    //--------------------------------------------
    public int FindIndex(String Key) {
    //--------------------------------------------
    // Returns -1 for not found
    int N, Ix0;
    TListItem AListItem;
    
      for (N = 0; N < ArgCount(); N++) {
        AListItem = (TListItem) AList[N];
        if (String.Compare(Key, AListItem.Key, true) == 0 ) { // ignore case
          Ix0 = N;
          goto foundit;
        }
      }
      Ix0 = -1;
    foundit: 
      return Ix0;
    }

    //--------------------------------------------
    public String Keys(int Ix0) {
    //--------------------------------------------
    // Returns empty if not found
      String Result = "";
      if (NETVSL_Util.WithinBounds(Ix0, 0, ArgCount()-1 ) ) {
        TListItem AListItem = (TListItem) AList[Ix0];
        Result = AListItem.Key;
      }  
      return Result;
    }

    //--------------------------------------------
    public String Values(int Ix0) {
    //--------------------------------------------
    // Returns empty if not found
      String Result = "";
      if (NETVSL_Util.WithinBounds(Ix0, 0, ArgCount()-1 ) ) {
        TListItem AListItem = (TListItem) AList[Ix0];
        Result = AListItem.Value;
      }  
      return Result;
    }

    //--------------------------------------------
    public String FindValue(String Key) {
    //--------------------------------------------
    // Returns empty if not found
      String Result = "";
      int Ix0 = FindIndex(Key);
      if (Ix0 != -1) {
        Result = Values(Ix0);
      }  
      return Result;
    }    
    
	} // class
} // namespace
