//--------------------------------------------------
// NETVSL_Types.cs
// Part of Diagramantics NETVSL .NET-to-VSL interface for Visio
// Provided for use with the book:
// Microsoft Visio 2003 Developer's Survival Pack
// Copyright 2003
// Graham Wideman
// Permission granted to owners of the book to incorporate into your own 
// code, and to deploy that code freely.
// 
// -----------------------------
// Purpose:
// Defines the INETVSL Interface for .NET, from which is derived the 
// INETVSL Interface for COM.
// -----------------------------
//
// ----------------------------------------------
//     I M P O R T A N T
// ----------------------------------------------
// -- Do not modify this file. It must stay in sync with the 
//    NETVSL Adapter's idea of what these interfaces and enums look like.
//
// -- Simply include this file it in your C# project, by adding a 
//    *Link* to this file in your project in the Solution Explorer. 
//    Ie: Use Add > Add Existing Item and in the open-file dialog
//    use the Open button's drop down to select Link. It's fiddly,
//    you have to select but not double click the filename first.
//
// -- See samples to see how to base your own main NETVSL implementation 
//    class on INETVSL.
//
// -----------------------------------------------
//
// Revision History:
// 
// 2004-02-14 GW First release version
//
// 2003-10-xx through 
// 2004-02-xx GW W-a-a-a-y too much messing around trying to understand 
//               COM Interop. 
//
//------------------------------------------------

using System;    
using System.Runtime.InteropServices;

namespace NETVSL_Types {

  public enum NETVSL_ResultCodes {
    GEN_INVBASE      = 0x00000000,
    GEN_SENBASE      = 0x00001000,
    L2V_INVBASE      = 0x00004000,
    L2V_SENBASE      = 0x00005000,
    SUCCESS          = 0x00000000,
    FAILURE          = 0x00001000,
    L2V_LOADFAILED   = 0x00004000,
    L2V_ENABLED      = 0x00005000,
    L2V_MODELESS     = 0x00005001,
    L2V_GOTNOABOUT   = 0x00005002,
    L2V_GOTNOHELP    = 0x00005003,
    L2V_BADMSG       = 0x00005005,
    L2V_VERSION      = 0x00000023
  };

  public enum NETVSL_AddOnInfo_Attribs {
    ATTS_ISACTION = 0x00000001,
    ATTS_WAITCRSR = 0x00000008,
    ATTS_DONTSHOW = 0x00000010
  }

  public enum NETVSL_AddOnInfo_EnableMask {
    ENABLEALWAYS     = 0x0000FFFF,
    ENABLEDYNAMIC    = 0x00000000,
    ENABLEDYNAMICEX  = 0x00008000,
    ND               = 0x00000001,
    NW               = 0x00000002,
    NDrW             = 0x00000004,
    NStW             = 0x00000008,
    NShW             = 0x00000010,
    NIcW             = 0x00000020,
    NT               = 0x00000040,
    NPgT             = 0x00000080,
    NMsT             = 0x00000100,
    NS               = 0x00000200,
    NEEDSDOC         = 0x00000001,
    NEEDSDOCWND      = 0x00000003,
    NEEDSDRAWWND     = 0x00000007,
    NEEDSPAGEWND     = 0x00000087,
    NEEDSMASTERWND   = 0x00000107,
    NEEDSSTENWND     = 0x0000000B,
    NEEDSSHEETWND    = 0x00000013,
    NEEDSICONWND     = 0x00000023,
    NEEDSTARGET      = 0x00000041,
    NEEDSTARGPAGE    = 0x000000C1,
    NEEDSTARGMASTER  = 0x00000141,
    NEEDSSELECTION   = 0x00000241
  }

  public enum NETVSL_AddOnInfo_InvokeOnMask {
    INVOKE_DEFAULT   = 0x00000000,
    INVOKE_LAUNCH    = 0x00000001
  }

  public enum NETVSL_AddOnInfo_NotifyOnMask {
    NOTIFY_DEFAULT   = 0x00000000
  }

  public enum NETVSL_AdapterMessage {
    None             = 0x00000000,
    OpenTrace        = 0x00000001,
    CloseTrace       = 0x00000002,
    ClearMessages    = 0x00000003,
    AddMessage       = 0x00000004,
    AddMessageWShow  = 0x00000005
  }

  [InterfaceTypeAttribute(ComInterfaceType.InterfaceIsDual)]
  [GuidAttribute("98F04C47-3B2C-4682-B930-0C767B4C23B8")]  // fresh GUID
  //======================================
  public interface INETVSL {
  //======================================
    //----------------------------------------
    NETVSL_ResultCodes Load (Int32 InterfaceVersion);

    //----------------------------------------
    int AddonCount {
      get;
    }

    //----------------------------------------
    NETVSL_ResultCodes GetAddonInfo ( 
      Int32 SessionId , 
      Int32 AddOnIx , 
      ref String Name , 
      ref NETVSL_AddOnInfo_Attribs        Attribs , 
      ref NETVSL_AddOnInfo_EnableMask     EnableMask , 
      ref NETVSL_AddOnInfo_InvokeOnMask   InvokeOnMask , 
      ref NETVSL_AddOnInfo_NotifyOnMask   NotifyOnMask );

    //----------------------------------------
    NETVSL_ResultCodes IsAddOnEnabled ( 
      System.Int32 AddOnIx , System.Int32 SessionId );
  
    //----------------------------------------
    Int32 PollNextEndSessionMessage (ref Int32 SessionId );

    //----------------------------------------
    Int32 PollNextTraceMessage (ref NETVSL_AdapterMessage MsgType , 
      ref String Text );

    //----------------------------------------
    void KillSession ( System.Int32 SessionId );

    //----------------------------------------
    NETVSL_ResultCodes Run ( 
      Int32   AddOnIx , 
      Int32   SessionId , 
      String  Args );

    //----------------------------------------
    NETVSL_ResultCodes Unload (  );

    //----------------------------------------
    Int32 Test ( Int32 I );

    //----------------------------------------
    void VSLInfo (ref String Descr );

  }
}
