/*-------------------------------------
This is a simple example addon based on TNETVSL_Addon.
It demonstrates just the minimal pieces you need to implement
to have a working Addon if you are using this scheme.

Basic functionality:
--------------------
-- Initializes the name and options in the constructor.

-- Run method. 
   -- Shows args in a Visio shape
   -- Looks for CMD arg, and optionally shows message box.

--------------------------------------*/

using System;
using NETVSL_Types;
using NETVSL_Base;
using NETVSL_Common;

using System.Windows.Forms;
using Visio = Microsoft.Office.Interop.Visio;

namespace NVTest_TwoAddons
{
	//----------------------------------------
	internal class TNVTest_TwoAddons_Addon01 : TNETVSL_Addon
	//----------------------------------------
	{
	  //------------------------------------
	  // Init name and behavior options for this Addon 
	  // in the call to base constructor	
	  //------------------------------------
		public TNVTest_TwoAddons_Addon01(TNETVSL_Base AOwner):
	  //------------------------------------	  
		  base(AOwner, 
		       "NVTest_TwoAddons_Addon01"               ,
           NETVSL_AddOnInfo_Attribs.ATTS_ISACTION   , 
           NETVSL_AddOnInfo_EnableMask.ENABLEALWAYS , 
           NETVSL_AddOnInfo_InvokeOnMask.INVOKE_DEFAULT , 
           NETVSL_AddOnInfo_NotifyOnMask.NOTIFY_DEFAULT  )      
		{
      // nothing more to do for this Addon
		}
		
		
	  //-------------------------------------------------
	  // Run function is the only other piece	required to 
	  // be implemented
	  //------------------------------------------------
		
    //----------------------------------------
    public override NETVSL_ResultCodes Run (Int32   SessionId , String  Args ) 
    //----------------------------------------
    {
      TArgHandler ArgHandler = new TArgHandler();
      String ArgStrForShape = "Args: \n";
      
      //----------------------------------------
      // Parse Args, prep to write to Shape.Text
      //----------------------------------------      
      ArgHandler.ParseArgs(Args);            
      
      for (int Ix0 = 0; Ix0 < ArgHandler.ArgCount(); Ix0++) {
        ArgStrForShape = ArgStrForShape 
                       + ArgHandler.Keys(Ix0) + " = " 
                       + ArgHandler.Values(Ix0) 
                       + "\n";      
      }
      
      //-------------------------------------------
      // Find first shape on active page and 
      // place args string into it
      //-------------------------------------------
      Visio.Page        APage   = null;
      Visio.Shape       AShape  = null;
    
      APage  = OwnerVSL.VisApp.ActivePage;
      AShape = APage.Shapes[1];

      if (AShape == null) goto done;
      
      AShape.Text = ArgStrForShape; 
      
      //-------------------------------------------
      // Check for a CMD parameter, and see if it
      // requests a message box.
      //-------------------------------------------
      String Cmd = ArgHandler.FindValue("CMD");
      if (Cmd == "MsgBox") {
        MessageBox.Show("Here is a message box");      
      }
      
    done:       
      return NETVSL_ResultCodes.SUCCESS; 
    }	
	}
}
